@@ -806,6 +806,30 @@ cc_wwan_data_activated_cb (GObject      *object,
     g_warning ("Error: %s", error->message);
 }
 
+static void
+cc_wwan_data_disconnect_cb (GObject      *object,
+                            GAsyncResult *result,
+                            gpointer      user_data)
+{
+  CcWwanData *self;
+  g_autoptr(GTask) task = user_data;
+  g_autoptr(GError) error = NULL;
+
+  self = g_task_get_source_object (G_TASK (task));
+  if (nm_device_disconnect_finish (self->nm_device, result, &error))
+    {
+      g_clear_object (&self->active_connection);
+      g_task_return_boolean (task, TRUE);
+    }
+  else
+    {
+      g_task_return_error (task, g_steal_pointer (&error));
+    }
+
+  if (error)
+    g_warning ("Error: %s", error->message);
+}
+
 static void
 cc_wwan_data_settings_saved_cb (GObject      *object,
                                 GAsyncResult *result,
@@ -838,15 +862,10 @@ cc_wwan_data_settings_saved_cb (GObject      *object,
     }
   else
     {
-      if (nm_device_disconnect (self->nm_device, cancellable, &error))
-        {
-          g_clear_object (&self->active_connection);
-          g_task_return_boolean (task, TRUE);
-        }
-      else
-        {
-          g_task_return_error (task, g_steal_pointer (&error));
-        }
+      nm_device_disconnect_async (self->nm_device,
+                                  cancellable,
+                                  cc_wwan_data_disconnect_cb,
+                                  g_steal_pointer (&task));
     }
 }
 
